'
'   sdSpeedTest.spin
'   a short program that uses the block layer methods of fsrw 2.6 to test the
'   speed at which data can be written to the SD card.
'
'   pgb 15 Dec 2011
'  
CON
  _clkmode = xtal1 + pll16x                      ' Low speed external crystal 16x PLL
  _xinfreq = 5_000_000

  SD_DO  = 0
  SD_CLK = 1
  SD_DI  = 2
  SD_CS  = 3

  LONGS_IN_BUFFER   = 256
  BUFFERS_TO_WRITE   = 10_000 ' (SPS*CHANNELS*RECORD_LENGTH)/LONGS_PER_BLOCK
  

  FILL_STATUS       = 25  ' pin to monitor
  WRITE_STATUS      = 26  ' pin to monitor
  READ_STATUS       = 27  ' pin to monitor


  CR                = 13      ' ASCII <CR>
  LF                = 10      ' ASCII <LF>
  SPACE             = 32      ' ASCII <SPACE>
  DOT               = 46      ' ASCII .
  TAB               = 09      ' ASCII <TAB>

  DEBUG             = 0
  DEBUG_BAUD        = 38400

OBJ
  uarts    : "pcFullDuplexSerial4FC"               '1 COG for 3 serial ports
  Num      : "Numbers"                             'Include Numbers object for writing numbers to debuginal
  block    : "safe_spi"

VAR
  byte mainCogID, serialCogID, firCogID, sdCardID
  long inputBuffer[LONGS_IN_BUFFER], outputBuffer[LONGS_IN_BUFFER]
  long fileSize
  long sumWrite, maxWrite, minWrite, sumRead, maxRead, minRead

PUB MAIN | i, b1, e1, b2, e2, blockNum, readPtr, offset, rTime, wTime

  PAUSE_MS(200)

  Num.Init
  uarts.Init
  uarts.AddPort(DEBUG,31,30,-1,-1,0,0,DEBUG_BAUD) 'Add DEBUG port

  PAUSE_MS(200)
  mainCogId   := cogid
  serialCogId := uarts.Start - 1                  'Start the ports

  i := block.start_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)

  PAUSE_MS(2000)
  uarts.putc(DEBUG,CR)
  uarts.putc(DEBUG,CR)
  uarts.putc(DEBUG,CR)
  uarts.str(DEBUG,string(13,"Main Cog:                        "))
  uarts.dec(DEBUG,mainCogId)
  uarts.str(DEBUG,string(13,"Serial ports running in cog:     "))
  uarts.dec(DEBUG,serialCogId)
  uarts.str(DEBUG,string(13,"Mount Command Returned:          "))
  uarts.dec(DEBUG,i)

  
  uarts.str(DEBUG,string(13,"Blocks to write: "))
  uarts.dec(DEBUG,BUFFERS_TO_WRITE)
  
  uarts.str(DEBUG,string(13,"uSeconds to init inputBuffer and outputBuffer: "))
  b1:= cnt
  longfill(@inputBuffer,  0, LONGS_IN_BUFFER)                                                                                          
  longfill(@outputBuffer, 0, LONGS_IN_BUFFER)
  e1:= cnt
  uarts.dec(DEBUG,(e1-b1)/80)
  uarts.putc(DEBUG,CR)
  uarts.putc(DEBUG,CR)
  

  dira[FILL_STATUS]   := 1
  dira[WRITE_STATUS]  := 1
  dira[READ_STATUS]   := 1
  outa[FILL_STATUS]   := 0
  outa[WRITE_STATUS]  := 0
  outa[READ_STATUS]   := 0

' init some values
  blockNum :=    0
  offset   := 1240
  readPtr  := 8934
  sumWrite := 0
  maxWrite := 0
  minWrite := 876543210  ' some big number
  sumRead  := 0
  maxRead  := 0
  minRead  := 876543210  ' some big number
    
  uarts.str(DEBUG,string(13,"Running read and write speed tests."))
  b1 := e1 := e2 := cnt
  repeat i from 0 to (BUFFERS_TO_WRITE - 1) step 2
    b2 := cnt
    blockNum+=2
    longfill(@outputBuffer, (e2-b2), LONGS_IN_BUFFER)
    wTime:=WRITE_BUFFER(offset+i,@outputBuffer)

    longfill(@inputBuffer, 0, LONGS_IN_BUFFER )
    rTime:=READ_BUFFER(readPtr+i,@inputBuffer)

    e2:=cnt
  e1 := cnt
    
  uarts.str(DEBUG,string(13,"block",TAB,TAB))
  uarts.decf(DEBUG,i,9)
  uarts.str(DEBUG,string(13,"offset",TAB,TAB))
  uarts.decf(DEBUG,offset,9)
  uarts.str(DEBUG,string(13,"readPtr",TAB,TAB))
  uarts.decf(DEBUG,readPtr,9)
  uarts.str(DEBUG,string(13,"time(us)",TAB))
  uarts.decf(DEBUG,(e1-b1)/80,9)

  uarts.putc(DEBUG,13)
  uarts.str(DEBUG,string(13,"sumWrite(us)",TAB))
  uarts.decf(DEBUG,(sumWrite),9)
  uarts.str(DEBUG,string(13,"maxWrite(us)",TAB))
  uarts.decf(DEBUG,(maxWrite)/80,9)
  uarts.str(DEBUG,string(13,"minWrite(us)",TAB))
  uarts.decf(DEBUG,(minWrite)/80,9)

  uarts.putc(DEBUG,13)
  uarts.str(DEBUG,string(13,"sumRead(us)",TAB))
  uarts.decf(DEBUG,(sumRead),9)
  uarts.str(DEBUG,string(13,"maxRead(us)",TAB))
  uarts.decf(DEBUG,(maxRead)/80,9)
  uarts.str(DEBUG,string(13,"minRead(us)",TAB))
  uarts.decf(DEBUG,(minRead)/80,9)

  uarts.putc(DEBUG,13)
  uarts.str(DEBUG,string(13,"Finished. Blocks processed: "))
  uarts.decf(DEBUG,blockNum,9)

PUB WRITE_BUFFER(blockNumber,dataPtr) | time, s, e
  s := cnt
  outa[WRITE_STATUS] := 1    
  block.writeblock(blockNumber,  dataPtr)         ' lower 512 bytes of buffer
  block.writeblock(blockNumber+1,dataPtr+512)     ' upper 512 bytes of buffer
  outa[WRITE_STATUS] := 0    
  e    := cnt
  time := e - s
  
  sumWrite+=(time/80)
  
  if time>maxWrite
    maxWrite := time
  
  if time<minWrite
    minWrite := time

  return time

PUB READ_BUFFER(blockNumber,dataPtr) | time, s, e
  s := cnt
  outa[READ_STATUS] := 1    
  block.readblock(blockNumber,  @inputBuffer)     ' lower 512 bytes of buffer
  block.readblock(blockNumber+1,@inputBuffer+512) ' upper 512 bytes of buffer
  outa[READ_STATUS] := 0
  e    := cnt
  time := e - s

  sumRead+=(time/80)
  
  if time>maxREad
    maxRead := time
  
  if time<minRead
    minRead := time

  return time

PUB PAUSE_MS(mS)
  waitcnt(clkfreq/1000 * mS + cnt)
  return

'      MIT LICENSE
{{
'  Permission is hereby granted, free of charge, to any person obtaining
'  a copy of this software and associated documentation files
'  (the "Software"), to deal in the Software without restriction,
'  including without limitation the rights to use, copy, modify, merge,
'  publish, distribute, sublicense, and/or sell copies of the Software,
'  and to permit persons to whom the Software is furnished to do so,
'  subject to the following conditions:
'
'  The above copyright notice and this permission notice shall be included
'  in all copies or substantial portions of the Software.
'
'  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
'  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
'  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
'  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
'  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
'  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
'  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
}}